------------------
--LOAD LIBRARIES--
------------------

--load LibAddonsMenu-2.0
local LAM2 = LibStub:GetLibrary("LibAddonMenu-2.0");

----------------------
--INITIATE VARIABLES--
----------------------

--create FenceData table
FenceData = {};
--define name of addon
FenceData.name = "FenceData";
--define addon version number
FenceData.version = 1.07;
--setting default settings for addon
FenceData.Default = {OffsetX = 0, OffsetY = 0, GUIvisible = true, compatMode = false};

--------------------
--SAVING VARIABLES--
--------------------

--function to activate when mouse is released from addon, triggering a save of variables
function FenceData.OnMouseUp()  
  --changing saved variables based upon where the GUI is moved
  FenceData.savedVariables.OffsetX = FenceDataUI:GetLeft();
  FenceData.savedVariables.OffsetY = FenceDataUI:GetTop();
end

---------------------------------
--ADDON SETTINGS MENU FUNCTIONS--
---------------------------------

local function CreateSettingsWindow()
  --create table to hold all panel information
  local panelData = {
    --telling LibAddonMenu that we have a panel
    type = "panel",
    --name = This is the name of the panel that will be displayed in the addon selection area of the settings menu
    name = FenceData.name,
    --name displayed in the addon settings menu
    displayName = FenceData.name,
    --display author name in addon settings menu
    author = "DaveC524",
    --display version number of addon in addon settings menu
    version = tostring(FenceData.version),
    --whether libAddonsMenu will refresh all options controls when a setting is changed
    registerForRefresh = true,
    --whether "Reset to Defaults" button exists and does things
    registerForDefaults = true,
    --function activated by "Defaults" button
    resetFunc = FenceData.reset,
  }
  
  --registering "panelData" settings panel
  local cntrlOptionsPanel = LAM2:RegisterAddonPanel("FenceDataConfigPanel", panelData);
  
  local optionsData = 
  {
    [1] = {type = "header", name = "Manage GUI"},
    [2] = {type = "description", text = "Use toggle to either show or hide the on-screen GUI"},
    [3] = {
      type = "checkbox", 
      name = "Show GUI", 
      tooltip = "Show/Hide", 
      default = true,
      --return the value of whether the GUI is visible or not
      getFunc = function() return FenceData.savedVariables.GUIvisible end,
      --when checkbox is checked, make visible or invisible the GUI
      setFunc = function(newValue)
        FenceData.savedVariables.GUIvisible = newValue;
        FenceDataUI:SetHidden(not newValue) end,
    },
    [4] = {
      type = "button", 
      name = "Reset GUI position", 
      tooltip = "CLICK ME!",
      --call function FenceData.resetGUIpos on button press
      func = FenceData.resetGUIpos, 
      width = "half",
    },
    [5] = {type = "header", name = "Manage Chat"},
    [6] = {type = "description", text = "Use to fix possible conflicts with other addons"},
    [7] = {
      type = "checkbox",
      name = "Enable or disable compatibility mode",
      tooltip = "Enable/Disable",
      --compat mode is off by default
      default = false,
      --return the value of whether compat mode is on or not
      getFunc = function() return FenceData.savedVariables.compatMode end,
      --when checkbox is checked, turn compat mode on or off
      setFunc = function(newValue)
        FenceData.savedVariables.compatMode = newValue;
        end,
    },
  } 
  --registering "optionsData" options panel
  LAM2:RegisterOptionControls("FenceDataConfigPanel", optionsData);
end

--function to reset everything
function FenceData.reset()
  --reset the position of the onscreen GUI
  FenceData.resetGUIpos();
  --reset compat mode
  FenceData.savedVariables.compatMode = FenceData.Default.compatMode;
end


--function to reset the GUIs position on the screen
function FenceData.resetGUIpos()
  --reset OffsetX variable
  FenceData.savedVariables.OffsetX = FenceData.Default.OffsetX;
  --reset OffsetY variable
  FenceData.savedVariables.OffsetY = FenceData.Default.OffsetY;
  --reset anchors
  FenceDataUI:ClearAnchors();
  --move to center of screen
  FenceDataUI:SetAnchor(CENTER, GuiRoot, CENTER);
  --make visible
  FenceData.savedVariables.GUIvisible = true;
  FenceDataUI:SetHidden(false)
end;

---------------------
--TRIGGER FUNCTIONS--
---------------------

--event triggered on addons loaded; function to sort through all incoming loaded addons, and only call function for the correct one
local function triggerAddonLoaded(eventCode, addonName)
  if  (addonName == FenceData.name) then
    --initial draw and refresh of UI
    FenceData.Update();
   
    --create new SavedVars for table, using default variables
    FenceData.savedVariables = ZO_SavedVars:New("FenceDataVars", FenceData.version, nil, FenceData.Default);
   
    --clear UI position
    FenceDataUI:ClearAnchors();    
    --if offsets are 0 (new installation of addon), place UI in center of screen, otherwise, place UI where it should be
    if(FenceData.savedVariables.OffsetX == FenceData.Default.OffsetX) and (FenceData.savedVariables.OffsetY == FenceData.Default.OffsetY) then
      --place addon in center of screen
      FenceDataUI:SetAnchor(CENTER, GuiRoot, CENTER);
    else
      --place UI where it was last remembered
      FenceDataUI:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, FenceData.savedVariables.OffsetX, FenceData.savedVariables.OffsetY);
    end
    
    --if GUI is set to be hidden, then hide
    if(FenceData.savedVariables.GUIvisible == false) then
      FenceDataUI:SetHidden(true)
    end
    
    --create addon settings window
    CreateSettingsWindow();
    
    --unregister initialization event
    EVENT_MANAGER:UnregisterForEvent(FenceData.name, EVENT_ADD_ON_LOADED);
  end
end

--event triggered by EVENT_JUSTICE_STOLEN_ITEMS_REMOVED
local function triggerStolenItemsRemoved(eventCode)
  FenceData.Update();
end

--event triggered when any slot in inventory updates
local function triggerSlotUpdate(eventCode)
  FenceData.Update();
end

--------------------
--PRINTING TO CHAT--
--------------------

--print sells left for the day
local function printFence(sellsUsed, totalSells)
  CHAT_SYSTEM:AddMessage("|cffffffTotal sells: |r" .. sellsUsed .. "|cffffff  of |r" .. totalSells .. "|cffffff sells used today|r");
end

--print launders left for the day
local function printLaunder(laundersUsed, totalLaunders)
  CHAT_SYSTEM:AddMessage("|cffffffTotal launders: |r" .. laundersUsed .. "|cffffff of |r" .. totalLaunders .. "|cffffff launders used today|r");
end

--function to print the pulled time into the proper format
local function printTime(seconds, minutes, hours)
  local time = (hours .. "|cffffff hours, |r" .. minutes .. "|cffffff minutes, |r" .. seconds .. "|cffffff seconds|r");
  CHAT_SYSTEM:AddMessage(time .. "|cffffff left until reset|r");
end

--function to print the total number of stolen items in bag and their total gold value
local function printNumStolenAndValue(totalNumStolen, totalValue)
  --get font size to size icons properly
  local fontSize = GetChatFontSize();
  
  --check if compat mode
  if (FenceData.savedVariables.compatMode == true) then
    --compat mode on
    CHAT_SYSTEM:AddMessage("|cffffffStolen items in bag: |r" .. totalNumStolen .. "|cffffff(|r" .. totalValue .. "|cffffff)|rg");
  else
    --compat mode off (no icons)
    CHAT_SYSTEM:AddMessage("|cffffffStolen items in bag: |r" .. totalNumStolen .. "|tfontSize:fontSize:esoui/art/inventory/inventory_stolenitem_icon.dds|t |cffffff(|r" .. totalValue .. "|tfontSize:fontSize:esoui/art/currency/currency_gold_32.dds|t|cffffff)|r");
  end
end

----------------------------------
--PULL/DATA PROCESSING FUNCTIONS--
----------------------------------

--function to convert time from seconds (pulled from API) to seconds, minutes, hours
local function convertTime(seconds)
  --initialize new variables
  local minutes, hours = 0, 0;
  --control for odd server bugs (negative time or above 24 hours)
  if (seconds < 0) or (seconds >= 86400) then
    seconds, minutes, hours = 0, 0, 0;
  --assuming nothing is going wrong, continue
  else  
    minutes = math.floor(seconds / 60);
    hours = math.floor(minutes / 60);
  
    if seconds >= 60 then
     seconds = seconds % 60;
    end
  
    if minutes >= 60 then
      minutes = minutes % 60;
    end
  end
  --seconds, minutes, and hours are returned separately
  return seconds, minutes, hours;
end

--function to pull fence data (max daily sells, daily sells used, and time until reset in seconds)
local function pullFence ()
  local totalSells, sellsUsed, resetTimeSeconds = GetFenceSellTransactionInfo();
  return totalSells, sellsUsed, resetTimeSeconds;
end

--function to pull launder data (max daily launders, daily launders used, and time until reset in seconds)
local function pullLaunder()
  local totalLaunders, laundersUsed, resetTimeSeconds = GetFenceLaunderTransactionInfo();
  return totalLaunders, laundersUsed, resetTimeSeconds;
end

--function to get the fencing value of each stolen item, including bonuses and stick size
local function getSellValue(bagId, slotIndex, quantity)
  local sellValue = (GetItemSellValueWithBonuses(bagId, slotIndex)) * quantity;
  return sellValue;
end

--function to scan bag, looking for stolen items, returning total value and total number of stolen goods
local function scanStolen()
  --find out total bag size of the user
  local bagsize = GetBagSize(1);
  
  --initialize local variables and reset to 0;
  local i, numStolen, totalNumStolen, itemValue, totalValue = 0, 0, 0, 0, 0;
  
  while (i <= bagsize) do
    if (IsItemStolen(1, i)) == true then
      --calculate number of stolen items per slot
      numStolen = (GetSlotStackSize(1, i));   
      --add to the total number of stolen items
      totalNumStolen = totalNumStolen + numStolen;  
      --calculate item value per stack of stolen items
      itemValue = getSellValue(1, i, numStolen);
      --add to total value of stolen goods
      totalValue = totalValue + itemValue;
    end
    --increment loop
    i = i + 1;
  end
  --return values
  return totalNumStolen, totalValue;
end

------------------
--CHAT FUNCTIONS--
------------------

--function activated by "/fence"
local function fence()
  --pulling fence data and storing
  local totalSells, sellsUsed, resetTimeSeconds = pullFence();
  --pulling launder data and storing
  local totalLaunders, laundersUsed, resetTimeSeconds = pullLaunder();
  --find out how many stolen items are in player inventory, and calculate fence selling return
  local totalNumStolen, totalValue = scanStolen();
  
  --renaming variable and cleaning up old variable for efficiency sake
  local seconds = resetTimeSeconds;
  resetTimeSeconds = nil;
 
  --call convertTime to change seconds into seconds, minutes, hours
  local seconds, minutes, hours = convertTime(seconds);
  
  --call print functions to spam chat with info
  printFence(sellsUsed, totalSells);
  printLaunder(laundersUsed, totalLaunders);
  printNumStolenAndValue(totalNumStolen, totalValue);
  printTime(seconds, minutes, hours);
end

------------
--DRAW GUI--
------------

--function to update UI 
function FenceData.Update()
  --call bagscanning algorithm, returning total number and value of stolen goods  
  local totalNumStolen, totalValue = scanStolen();
  
  --get fencing data (total sells and max sells per day); get laundering data (total launders and max launders per day)
  local totalSells, sellsUsed = pullFence();
  local totalLaunders, laundersUsed = pullLaunder();
  
  --get font size to size icons properly
  local fontSize = GetChatFontSize();
  --update UI boxes with properly scaled icons and proper data
  FenceDataUIBox1:SetText(string.format("|tfontSize:fontSize:esoui/art/vendor/vendor_tabicon_sell_up.dds|t: |cffffff%d|r of |cffffff%d|r", sellsUsed, totalSells));
  FenceDataUIBox2:SetText(string.format("|tfontSize:fontSize:esoui/art/vendor/vendor_tabicon_fence_up.dds|t: |cffffff%d|r of |cffffff%d|r", laundersUsed, totalLaunders));
  FenceDataUIBox3:SetText(string.format("|tfontSize:fontSize:esoui/art/inventory/inventory_tabicon_items_up.dds|t: |cffffff%d|r|tfontSize:fontSize:esoui/art/inventory/inventory_stolenitem_icon.dds|t (|cffffff%d|r|tfontSize:fontSize:esoui/art/currency/currency_gold_32.dds|t)", totalNumStolen, totalValue));
end

----------------------
--EVENT REGISTRATION--
----------------------

--register the slash command "/fence" to allow user command to do things
SLASH_COMMANDS["/fence"] = fence
--register events to update addon when loaded, when stolen items are removed while arrested, or when any item slot is updated
EVENT_MANAGER:RegisterForEvent(FenceData.name, EVENT_ADD_ON_LOADED, triggerAddonLoaded);
EVENT_MANAGER:RegisterForEvent(FenceData.name, EVENT_JUSTICE_STOLEN_ITEMS_REMOVED, triggerStolenItemsRemoved);
EVENT_MANAGER:RegisterForEvent(FenceData.name, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, triggerSlotUpdate)
